package SPVM::Net::SSLeay::Error::PEM_R_NO_START_LINE;



1;

=head1 Name

SPVM::Net::SSLeay::Error::PEM_R_NO_START_LINE - The Error indicated by PEM_R_NO_START_LINE in OpenSSL

=head1 Description

Net::SSLeay::Error::PEM_R_NO_START_LINE class in L<SPVM> represents the error indicated by L<PEM_R_NO_START_LINE|https://docs.openssl.org/master/man3/PEM_R_NO_START_LINE> in OpenSSL.

=head1 Usage

  use Net::SSLeay::Error::PEM_R_NO_START_LINE;

=head1 Super Class

L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error>

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License

